# Generated by Django 2.0 on 2018-04-18 09:12

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ('security', '0008_auto_20180306_2254'),
    ]

    operations = [
        migrations.AlterField(
            model_name='inputloggedrequest',
            name='status',
            field=models.PositiveSmallIntegerField(
                choices=[(0, 'Incomplete'), (20, 'Info'), (30, 'Warning'), (40, 'Error'), (10, 'Debug'),
                         (50, 'Critical')], default=0, verbose_name='status'),
        ),
        migrations.AlterField(
            model_name='outputloggedrequest',
            name='input_logged_request',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL,
                                    related_name='output_logged_requests', to='security.InputLoggedRequest',
                                    verbose_name='input logged request'),
        ),
        migrations.AlterField(
            model_name='outputloggedrequest',
            name='status',
            field=models.PositiveSmallIntegerField(
                choices=[(0, 'Incomplete'), (20, 'Info'), (30, 'Warning'), (40, 'Error'), (10, 'Debug'),
                         (50, 'Critical')], default=0, verbose_name='status'),
        ),
    ]
