# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2019-07-10 16:19
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('security', '0009_auto_20180418_0912'),
    ]

    operations = [
        migrations.AlterField(
            model_name='inputloggedrequest',
            name='type',
            field=models.PositiveSmallIntegerField(
                choices=[(1, 'Common request'), (2, 'Throttled request'), (3, 'Successful login request'),
                         (4, 'Unsuccessful login request'), (5, 'Successful two factor code verification request'),
                         (6, 'Unsuccessful two factor code verification request')],
                default=1, verbose_name='type'),
        ),
    ]
